local addon = WakeSpams
local module = addon:NewModule("Spells")
WakeSpams.Spells = module

function module:InitSpellDB()
	local _, class = UnitClass("player");
	if not class then error("Error: WakeSpams was unable to retrieve player englishclass.") end
	
	--[[
		SpellList args:
		Spell ID {
			disabled = boolean
			name = string (optional) custom spell name
			txt = string (optional) message on start
			txtend = string (optional) message on end8
			dur = int (optional) spell duration
			countdown = int (optional) fade countdown in chat
			fade = boolean (optional) announce when spell fades, default = nil
			whisper = boolean, default = nil
			aoe = boolean, default = nil
			output = string (optional) output channel
			event = string (optional) event name
			configname = string (optional) name that will show up in the config
		}
	]]
	
	if(class == "DEATHKNIGHT") then
		addon.DefaultSpellDB = {
			[48792] = { -- Icebound Fortitude
				dur = 12,
				countdown = 0,
				fade = true,
			},
			[55233] = { -- Vampiric Blood
				dur = 15,
				countdown = 0,
				fade = true,
			},
			[56222] = { -- Dark Command taunt
				txt = "Taunt Failed: spell:link on spell:target (spell:extra)",
				event = "SPELL_MISSED",
			},
			[49560] = { -- Death Grip (taunt component)
				txt = "Taunt Failed: spell:link on spell:target (spell:extra)",
				event = "SPELL_MISSED",
			},
			[49016] = { -- Hysteria (whisper)
				txt = "Activated: spell:link on YOU - spell:durations!",
				dur = 30,
				whisper = true,
				countdown = 0,
				fade = true,
			},
			[47476] = { -- Strangulate
				txt = "spell:link: spell:target",
			},
		}
	end
	
	if(class == "DRUID") then
		addon.DefaultSpellDB = {
			[22812] = { -- Barkskin
				dur = 12,
				countdown = 0,
				fade = true,
			},
			[61336] = { -- Survival Instincts
				dur = 20,
				countdown = 0,
				fade = true,
			},
			[29166] = { -- Innervate (whisper)
				txt = "Activated: spell:link on YOU - spell:durations!",
				dur = 10,
				countdown = 0,
				fade = true,
				whisper = true,
			},
			[8983] = { -- Bash
				txt = "Activated: spell:link on spell:target!",
				countdown = 0,
				fade = true,
			},
			[48477] = { -- Rebirth
				txt = "spell:link on spell:target!",
				event = "SPELL_RESURRECT",
			},
		}
	end
	
	if(class == "HUNTER") then
		addon.DefaultSpellDB = {
			[34477] = { -- Misdirection (cast... Not tested)
				txt = "spell:link on spell:target!",
				event = "SPELL_CAST_SUCCESS",
				whisper = true,
			},
		}
	end
	
	if(class == "PALADIN") then
		addon.DefaultSpellDB = {
			[20236] = { -- Lay on Hands (holy phys. damage reduction only. Right spell id?)
				txt = "Activated: spell:link on spell:target - spell:durations!",
				dur = 15,
				countdown = 0,
				fade = true,
			},
			[31821] = { -- Aura Mastery
				dur = 6,
				countdown = 0,
				fade = true,
			},
			[498] = { -- Divine Protection
				dur = 12,
				countdown = 0,
				fade = true,
			},
			[70940] = { -- Divine Guardian (DiSac raid component)
				dur = 6,
				countdown = 0,
				fade = true,
			},
			[66235] = { -- Ardent Defender proc (Not tested)
				txt = "spell:link saved me! Healed spell:amount!",
				event = "SPELL_HEAL",
			},
			[10308] = { -- Hammer of Justice
				txt = "Activated: spell:link on spell:target!",
				countdown = 0,
				fade = true,
			},
			[48817] = { -- Holy Wrath
				countdown = 0,
				fade = true,
				aoe = true,
			},
			[1044] = { -- Hand of Freedom
				txt = "spell:link on YOU - spell:durations!",
				dur = 6,
				whisper = true,
			},
			[10278] = { -- Hand of Protection
				txt = "Activated: spell:link on spell:target - spell:durations!",
				dur = 10,
			},
			[6940] = { -- Hand of Sacrifice
				txt = "Activated: spell:link on spell:target - spell:durations!",
				dur = 12,
				countdown = 0,
				fade = true,
			},
			[1038] = { -- Hand of Salvation (whisper)
				dur = 10,
				txt = "Activated: spell:link on YOU - spell:durations!",
				countdown = 0,
				fade = true,
				whisper = true,
			},
		}
	end
	
	if(class == "PRIEST") then
		addon.DefaultSpellDB = {
			[65081] = { -- Body and Soul (target effect triggered by body and soul)
				txt = "spell:link on YOU - meep meep!",
				dur = 4,
				whisper = true,
			},
			[10890] = { -- Psychic Scream
				dur = 8,
				aoe = true,
			},
			[10060] = { -- Power Infusion (whisper)
				txt = "Activated: spell:link on YOU - spell:durations!",
				dur = 15,
				whisper = true,
				fade = true,
			},
			[64901] = { -- Hymn of Hope
				txt = "Activated: spell:link!",
				event = "SPELL_CAST_SUCCESS",
			},
			[47788] = { -- Guardian Spirit
				txt = "Activated: spell:link on spell:target - spell:durations!",
				dur = 10,
				countdown = 2,
				fade = true,
			},
			[48153] = { -- Guardian Spirit (heal)
				txt = "spell:link healed spell:target for spell:amount",
				event = "SPELL_HEAL",
				configname = "Guardian Spirit (heal effect)",
			},
			[33206] = { -- Pain Suppression
				txt = "Activated: spell:link on spell:target - spell:durations!",
				dur = 8,
				fade = true,
			},
			[64843] = { -- Divine Hymn
				txt = "Activated: spell:link!",
				event = "SPELL_CAST_SUCCESS",
			},
		}
	end
	
	if(class == "ROGUE") then
		addon.DefaultSpellDB = {
			[57934] = { -- Tricks of the Trade (cast... Not tested)
				txt = "spell:link on spell:target!",
				dur = 6,
				event = "SPELL_CAST_SUCCESS",
				whisper = true,
			},
			[1833] = { -- Cheap Shot
				txt = "Activated: spell:link on spell:target!",
				countdown = 0,
				fade = true,
			},
			[8643] = { -- Kidney Shot
				txt = "Activated: spell:link on spell:target!",
				countdown = 0,
				fade = true,
			},
		}
	end
	
	if(class == "SHAMAN") then
		addon.DefaultSpellDB = {
			[2825] = { -- Bloodlust
				txt = "Activated: spell:link! spell:duration SECONDS OF PURE FOCUS",
				dur = 40,
				countdown = 0,
				fade = true,
			},
			[32182] = { -- Heroism
				txt = "Activated: spell:link! spell:duration SECONDS OF PURE FOCUS",
				dur = 40,
				countdown = 0,
				fade = true,
			},
		}
	end
	
	if(class == "WARRIOR") then
		addon.DefaultSpellDB = {
			[871] = { -- Shield Wall
				dur = 12,
				countdown = 0,
				fade = true,
			},
			[12975] = { -- Last Stand
				dur = 20,
				countdown = 0,
				fade = true,
			},
			[46968] = { -- Shockwave
				txt = "Activated: spell:link!",
				countdown = 0,
				fade = true,
			},
			[12809] = { -- Concussion Blow
				txt = "Activated: spell:link on spell:target!",
				dur = 5,
				countdown = 0,
				fade = true,
			},
		}
	end
	
	if(class == "WARLOCK") then
		addon.DefaultSpellDB = {
			[1122] = { -- Summon Inferno
				txt = "Activated: spell:link!",
				countdown = 0,
				fade = true,
				event = "SPELL_CAST_SUCCESS",
			},
			[17928] = { -- Howl of Terror
				aoe = true,
			},
		}
	end
	
	addon.DefaultEventDB = {
		["SPELL_DISPEL"] = {
			txt = "spell:link Removed: spell:extra:link from spell:target",
		},
		["SPELL_DISPEL_FAILED"] = {
			txt = "spell:link Dispel Failed: spell:extra:link from spell:target",
		},
		["SPELL_INTERRUPT"] = {
			txt = "spell:link Interrupted: spell:target spell:extra:link",
		},
	}
	
	
	-- Helps the combat log scanner spell search by listing them by event. But... I dont even need to xDDD
	--addon:SetupEventHelper()
end

module:InitSpellDB()