--[[
##############################################
##-
##-  WakeSpams
##-  By Cyrila <Aspire> @ Bloodfeather-EU
##-
##-  Cooldowns For Your Raid
##-
##-
##-   * Copyright (C) 2009  Dan Jacobsen
##-   *
##-   * This program is free software: you can redistribute it and/or modify
##-   * it under the terms of the GNU General Public License as published by
##-   * the Free Software Foundation, either version 3 of the License, or
##-   * (at your option) any later version.
##-   *
##-   * This program is distributed in the hope that it will be useful,
##-   * but WITHOUT ANY WARRANTY; without even the implied warranty of
##-   * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##-   * GNU General Public License for more details.
##-   *
##-   * You should have received a copy of the GNU General Public License
##-   * along with this program.  If not, see <http://www.gnu.org/licenses/>.
##-
##############################################
]]

local WakeSpams = LibStub("AceAddon-3.0"):NewAddon("WakeSpams", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
_G["WakeSpams"] = WakeSpams

local _G, type, tonumber = _G, type, tonumber;
local floor = math.floor;
local wipe, tinsert, tremove, tsort = wipe, tinsert, tremove, table.sort;
local format, strsub, strfind, strlen, strformat, gsub = format, string.sub, string.find, string.len, string.format, string.gsub;
local pairs, ipairs  = pairs, ipairs;
local sin, ceil = math.sin, math.ceil;

local UnitName, GetSpellLink, UnitExists, UnitGUID, UnitHealthMax, GetUnitName = UnitName, GetSpellLink, UnitExists, UnitGUID, UnitHealthMax, GetUnitName;
local GetSpellInfo = _G["GetSpellInfo"];
local GetTime = _G["GetTime"];


-- ------------------------------------------------------------
--  INTERNAL VARIABLES (you might not want to touch these)
-- ------------------------------------------------------------

WakeSpams.Msg = nil
WakeSpams.Flag = 1
WakeSpams.Timer = {}
WakeSpams.LastSent = {0, ""}
WakeSpams.RaidIcons = {
	[1] = "{star} ", -- Yellow 4-point Star
	[2] = "{circle} ", -- Orange Circle
	[3] = "{diamond} ", -- Purple Diamond
	[4] = "{triangle} ", -- Green Triangle
	[5] = "{moon} ", -- White Crescent Moon
	[6] = "{square} ", -- Blue Square
	[7] = "{cross} ", -- Red 'X' Cross
	[8] = "{skull} "  -- White Skull
}
WakeSpams.Queue = {}
WakeSpams.Battlegrounds = {
	[1] = "Warsong Gulch",
	[2] = "Arathi Basin",
	[3] = "Alterac Valley",
	[4] = "Eye of the Storm",
	[5] = "Strand of the Ancients",
	[6] = "Isle of Conquest",
}


-- --------------------------------
--  DEFAULTS
-- --------------------------------

local db
local defaults = {
	profile = {
		disabled = false,
		Interval = 5,
		aoe = true,
		version = 4,
		AbilityStart = "Activated: spell:link spell:durations!",
		AbilityEnd = "Faded: spell:link",
		SpellDB = {},
		EventDB = {
			["SPELL_DISPEL"] = {
				disabled = true,
			},
		},
		OutputFlags = { -- If ChatType/Channel not available, say to SELF
			[1] = { -- Self
				"WHISPER", "TARGET"
			},
			[2] = { -- Party
				"PARTY", nil
			},
			[3] = { -- Raid
				"RAID", nil
			},
			[4] = { -- BG
				"WHISPER", "TARGET"
			},
			[5] = { -- WG/TB
				"WHISPER", "TARGET"
			},
			[6] = { -- Arena
				"PARTY", nil
			},
		},
	},
}

function WakeSpams:OnInitialize()
	-- Init
	self.db = LibStub("AceDB-3.0"):New("WakeSpamsDB", defaults, "Default")
	db = self.db.profile
	
	if(db.version ~= 4) then
		self.db:ResetProfile()
		--self.db.profile.version = 4
		self:Print("Profile reset to default.")
	end
	
	if not db.disabled then
		self:SetEnabledState(true)
	else
		self:SetEnabledState(false)
	end
	
	self:RegisterChatCommand("wakespams", "SlashHandler");
	self:RegisterChatCommand("ws", "SlashHandler");
end

function WakeSpams:OnEnable()
	-- ENABLED
	--self:Print(" -- WakeSpams Loaded! you must look, you must tab, you must see... future. -- ");
	
	-- Register events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "COMBAT_LOG_EVENT_UNFILTERED");
	self:RegisterEvent("RAID_ROSTER_UPDATE", "Event_RosterUpdate");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "Event_RosterUpdate");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "Event_ZoneChanged");
	--self:RegisterEvent("PLAYER_ENTERING_WORLD", "RAID_ROSTER_UPDATE");
end

function WakeSpams:OnDisable()
	-- Do something...
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:UnregisterEvent("RAID_ROSTER_UPDATE");
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED");
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA");
end


-- ----------------------------
--  Config
-- ----------------------------

function WakeSpams:SlashHandler(input)
	self.Options:ShowConfig()
	--[[
	local newsetting
	input = self:Explode(" ", string.lower(input))
	if(input[1] == "enable") then
		if(db.STANDBY == 0) then
			self:Print("Combat log monitoring is already enabled.")
		else
			db.STANDBY = 0
			self:Print("Combat log monitoring enabled.")
		end
	elseif(input[1] == "disable") then
		if(db.STANDBY == 1) then
			self:Print("Combat log monitoring is already disabled.")
		else
			db.STANDBY = 1
			self:Print("Combat log monitoring disabled.")
		end
	elseif(input[1] == "aoe") then
		db.aoe = (db.aoe == 0) and 1 or 0
		newsetting = (db.aoe == 0) and "Disabled" or "Enabled"
		self:Print(("AoE spell announcing has been >%s<"):format(newsetting))
	elseif(input[1] == "settings") then
		self.Options:ShowConfig()
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00WakeSpams help:")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Enable|r - Starts combat log monitoring")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Disable|r - Stops combat log monitoring")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Aoe|r - Toggles announcing multitarget spells (such as Shockwave and Howl of Terror)")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Help|r - Shows help")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC1100   This addon has no config. You can add and edit spell events in the Lua file.|r")
	end
	]]
end


-- ------------------------------------------------------------
--  HANDLE COMBAT LOG EVENTS AND PASS TO REGISTER SPELL OBJECT
-- ------------------------------------------------------------
function WakeSpams:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, sourceGUID, sourceName, _, destGUID, destName, _, spellid, spellname, _, arg1, arg2)
	-- Nil
	if(sourceGUID ~= UnitGUID("player")) then return nil end
	
	local spell
	
	-- Prioritize catchall events
	if(self.DefaultEventDB[event]) then
		if(db.EventDB[event]) then
			if(db.EventDB[event].disabled) then
				return nil
			end
		end
		spell = self.DefaultEventDB[event]
		spell.isEvent = true
		self.Msg = spell.txt or db.AbilityStart
	elseif(spellid) then
		-- The database does not exist
		if not self.DefaultSpellDB or not db.SpellDB then return nil end
		-- Continue if the spell exists in either the default spell list or in the saved variables
		if(self.DefaultSpellDB[spellid] or db.SpellDB[spellid]) then
			-- Get the spell table from the spell list if it exists, or
			-- In the case of a custom spell, since it will only exist in saved variables, fetch those
			if(self.DefaultSpellDB[spellid]) then
				spell = self.DefaultSpellDB[spellid]
				if(db.SpellDB[spellid]) then
					profile = db.SpellDB[spellid]
					if(profile.disabled) then return nil end
					spell.txt       = profile.txt       --or spell.txt
					spell.dur       = profile.dur       or 0 --or spell.dur
					spell.txtend    = profile.txtend    --or spell.txtend
					spell.countdown = profile.countdown or 0 --or spell.countdown
					spell.fade      = profile.fade      --or spell.fade
					spell.whisper   = profile.whisper   --or spell.whisper
				end
			else
				spell = db.SpellDB[spellid]
				if(spell.disabled) then return nil end
			end
			-- 
			if not db.aoe and (spell.aoe) then return nil end
			if not spell.event and (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REMOVED") then
				-- SPELL_AURA
				if(event == "SPELL_AURA_REMOVED") then
					if(spell.countdown and (spell.countdown > 0)) then
						-- Stop timers
						self:CancelTimer(self.Timer[spellid].timer, true)
						self.Timer[spellid] = nil
					end
					if not spell.fade then
						return nil
					end
					self.Msg = spell.txtend or db.AbilityEnd
				else
					self.Msg = spell.txt or db.AbilityStart
					
					if(self:isNumeric(spell.countdown) and spell.countdown > 0) then
						self.Timer[spellid] = {}
						self.Timer[spellid].elapsed = 0
						self.Timer[spellid].timer = self:ScheduleRepeatingTimer("TimerHandle", 1, {spellid, spell.dur, spell.dur-spell.countdown, destName}) -- duration, duration-time to know when to start announcing
					end
				end
			end
			if(spell.event == event) then
				self.Msg = spell.txt or db.AbilityStart
			end
		end
	end
	
	-- Return nil if no message was formatted
	if not self.Msg then return nil end
	
	if(spell.aoe) then
		self:Enqueue(spellid, destName)
		self.Msg = nil
		return true
	end
	
	--spellname = spell.name or GetSpellLink(spellid)
	msgframe = spell.output or self.MsgFrame
	
	-- Format string
	self.Msg = gsub(self.Msg, "spell:link", GetSpellLink(spellid) or "")
	self.Msg = gsub(self.Msg, "spell:name", spellname or "")
	self.Msg = gsub(self.Msg, "spell:duration", spell.dur or "")
	self.Msg = gsub(self.Msg, "spell:amount", tostring(arg1) or "")
	self.Msg = gsub(self.Msg, "spell:target", function(v)
		if(destName == UnitName("player")) then
			v = "me"
		else
			v = GetUnitName(destName, nil) or destName
			v = self:RaidIconText(self:GetUnitID(destGUID, destName))..v
		end
		return v
	end)
	self.Msg = gsub(self.Msg, "spell:extra:link", function(v)
		if(arg1 and (self:isNumeric(arg1))) then
			v = GetSpellLink(arg1) or ""
		end
		return v
	end)
	self.Msg = gsub(self.Msg, "spell:extra", function(v)
		if(event == "SPELL_MISSED") then
			v = tostring(arg1) or ""
		else
			v = arg2 or ""
		end
		return v
	end)
	
	spellid = (spell.isEvent) and event or spellid
	self:Announce(self.Msg, spellid, destName)
	-- Set msg to nil, otherwise we might have "some" spam issues.
	self.Msg = nil
end


-- ----------------------------
--  Misc. event handlers
-- ----------------------------
function WakeSpams:Event_RosterUpdate()
	self:UpdateFlag()
end

function WakeSpams:Event_ZoneChanged()
	-- Quote (WoWWiki):
	-- "When this event fires, the UI may still think you're in the zone you just left. Don't depend on GetRealZoneText()
	-- and similar functions to report the new zone in reaction to ZONE_CHANGED_NEW_AREA. (untested for similar events)"
	-- Circumvented by adding a delay to the function
	self:ScheduleTimer("UpdateFlag", 0.1)
end

function WakeSpams:UpdateFlag()
	-- 1: Solo, 2: Party, 3: Raid, 4: Battleground, 5: WG and Tol Barad, 6: Arena
	--[===[@debug@
	self:Print("Updating the State Flag.")
	--@end-debug@]===]
	if(IsActiveBattlefieldArena()) then
		self.Flag = 6
		--[===[@debug@
		self:Print("Set to: "..self.Flag)
		--@end-debug@]===]
		return true
	end
	if(GetNumRaidMembers() >= 1) then
		if(GetRealZoneText() == "Wintergrasp" or GetRealZoneText() == "Tol Barad") then
			self.Flag = 5
			--[===[@debug@
			self:Print("Set to: "..self.Flag)
			--@end-debug@]===]
			return true
		end
		for i,v in ipairs(self.Battlegrounds) do
			if(GetRealZoneText() == v) then
				self.Flag = 4
				--[===[@debug@
				self:Print("Set to: "..self.Flag)
				--@end-debug@]===]
				return true
			end
		end
		
		self.Flag = 3
		--[===[@debug@
		self:Print("Set to: "..self.Flag)
		--@end-debug@]===]
		return true
	end
	if(GetNumRaidMembers() <= 0 and GetNumPartyMembers() >= 1) then
		self.Flag = 2
		--[===[@debug@
		self:Print("Set to: "..self.Flag)
		--@end-debug@]===]
		return true
	end
	self.Flag = 1
	--[===[@debug@
	self:Print("Set to: "..self.Flag)
	--@end-debug@]===]
	return true
end


-- ---------------------------
--  Communication
-- ---------------------------

function WakeSpams:Enqueue(spellID, destName)
	if not self.Queue[spellID] then
		-- Create new timer
		self:ScheduleTimer("SendQueue", 0.2, spellID)
		self.Queue[spellID] = {}
	end
	tinsert(self.Queue[spellID], {destName});
end

function WakeSpams:SendQueue(spellID)
	if self.Queue[spellID] then
		local total
		local targetN
		for i,v in ipairs(self.Queue[spellID]) do
			total = i
			targetN = (targetN) and (targetN..", "..v[1]) or v[1]
		end
		local msg
		if(total == 1) then
			msg = (("%s: Hit %d target! (%s)"):format(GetSpellLink(spellID),total,targetN))
		else
			msg = (("%s: Hit %d targets!"):format(GetSpellLink(spellID),total))
		end
		--local msg = "SpellLink: Hit 3 targets (X, Y, Z)"
		self:Announce(msg, spellID)
		self.Queue[spellID] = nil
	end
end

function WakeSpams:TimerHandle(args)
	-- args: 1 = spellid, 2 = d, 3 = ts, 4 = target
	self.Timer[args[1]].elapsed = self.Timer[args[1]].elapsed + 1
	
	if(self.Timer[args[1]].elapsed >= args[3] and self.Timer[args[1]].elapsed < args[2]) then
		self:Announce(""..args[2] - self.Timer[args[1]].elapsed.."", args[1], args[4])
	end
end

function WakeSpams:Announce(msg, spellid, target)
	if(GetTime() < self.LastSent[1]+db.Interval and self.LastSent[2] == msg and self:isNumeric(spelllid)) then -- Temporary chat throttle
		return nil
	end
	
	local typeid, channel
	if(self:isNumeric(spellid)) then
		if(db.SpellDB[spellid] and (db.SpellDB[spellid].OutputFlags)) then
			if(db.SpellDB[spellid].whisper == true) then
				typeid, channel = "WHISPER", "TARGET"
			else
				if(db.SpellDB[spellid].OutputFlags[self.Flag]) then
					typeid, channel = db.SpellDB[spellid].OutputFlags[self.Flag][1], db.SpellDB[spellid].OutputFlags[self.Flag][2]
				else
					typeid, channel = db.OutputFlags[self.Flag][1], db.OutputFlags[self.Flag][2]
				end
			end
		else
			if(self.DefaultSpellDB[spellid].whisper) then
				typeid, channel = "WHISPER", "TARGET"
			else
				typeid, channel = db.OutputFlags[self.Flag][1], db.OutputFlags[self.Flag][2]
			end
		end
	else
		typeid, channel = db.OutputFlags[self.Flag][1], db.OutputFlags[self.Flag][2]
	end
	
	if(typeid == "DISABLE") then
		return true
	end
	if(typeid == "CHANNEL") then
		-- Resolve channel name
		channel, _, _ = GetChannelName(channel)
		if(channel == 0) then
			typeid = "SELF"
			self:Print("Custom channel name could not be resolved. You are not in that channel.")
		end
	end
	if(typeid == "WHISPER" and channel == "TARGET") then
		channel = target
		msg = gsub(msg, target, "YOU")
	end
	
	--[===[@debug@
	self:Print("Announce spell to: "..typeid or "nil"..", "..channel or "nil")
	--@end-debug@]===]
	
	self:ChatMSG(msg, typeid, nil, channel)
	self.LastSent = {GetTime(), msg}
end

function WakeSpams:ChatMSG(msg, mtype, mlang, marg)
	if(mtype=="SELF") then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	else
		if(mtype=="WHISPER" and marg==UnitName("player")) then
			DEFAULT_CHAT_FRAME:AddMessage(msg)
		else
			SendChatMessage(msg, mtype, mlang, marg)
		end
	end
end


-- ----------------------------
--  Helper functions
-- ----------------------------

function WakeSpams:ResetSpell(spellid)
	WakeSpams.Spells:InitSpellDB() -- Some strange bug in the config keeps changing the default spell db table, this is a cheap workaround
	self.db.profile.SpellDB[spellid] = self.DefaultSpellDB[spellid]
	self.db.profile.SpellDB[spellid].OutputFlags = {}
end

function WakeSpams:RaidIconText(t)
	local ri = ""
	if UnitExists(t) then
		ri = self.RaidIcons[GetRaidTargetIndex(t)] or ri
	end
	return ri
end

function WakeSpams:GetUnitID(GUID, name)
	-- Translates GUID to target (target, focus, mouseover, playername)
	if UnitGUID("focus") == GUID then
		return "focus"
	end
	if UnitGUID("mouseover") == GUID then
		return "mouseover"
	end
	if UnitGUID("target") == GUID then
		return "target"
	end
	return name
end

function WakeSpams:isNumeric(a)
    return type(tonumber(a)) == "number"
end

function WakeSpams:ApplyConstant(array)
	return setmetatable ({}, 
	{
		__index = array,
		__newindex = function (t, n, v)
			self:Print("Error: attempting to change constant "..tostring(n).." to "..tostring(v))
		end
    })
end

function WakeSpams:Explode(d, str)
	local t, ll
	t={}
	ll=0
	if(#str == 1) then return str end
		while true do
			l=strfind(str, d, ll+1, true)
			if l~=nil then
				tinsert(t, strsub(str, ll, l-1))
				ll=l+1
			else
				tinsert(t, strsub(str,ll))
				break
			end
		end
	return t
end